import numpy as np
import pickle
import os
from xlrd import open_workbook
import pandas as pd

# Marvalous tool api.
# evaluate
# predict

class Data:
    def __init__(self, text, rating):
        self.text = text
        self.rating = rating

class DataReaderWriter:
    @staticmethod
    def read_data(workbook_name):
        workbook = open_workbook(workbook_name)
        sheet = workbook.sheet_by_index(0)
        data = []
        print("Started reading from file ...")
        for cell_num in range(0, sheet.nrows):
            comments = Data(sheet.cell(cell_num, 0).value, sheet.cell(cell_num, 1).value)
            data.append(comments)

        return data


class Marvalous_tool:
    def __init__(self, marvalous_pickle_filepath):
        with open(marvalous_pickle_filepath, 'rb') as pkl_file:
            self.marvalous = pickle.load(pkl_file)
            pkl_file.close()

    def evaluate(self, test_filepath):
        test_data = np.array(DataReaderWriter.read_data(test_filepath))
        self.marvalous.evaluate(test_data=test_data)

    def predict(self, test_filepath):
        test_data = np.array(DataReaderWriter.read_data(test_filepath))
        return self.marvalous.predict(test_data=test_data)

if __name__ == "__main__":
    dest = os.path.join('../marvalous_tool', 'pkl_objects')

    marvalous_pickle_filename = 'marvalous.pkl'
    marvalous_70_perc_pickle_filename = 'marvalous_70_perc.pkl'

    test_filepath = "../data/test_30_perc/test.xlsx"

    marvalous_pickle_filepath = os.path.join(dest, marvalous_70_perc_pickle_filename)
    marvalous = Marvalous_tool(marvalous_pickle_filepath)

    print("MarValous: A tool for Emotional Analysis in Software Engineering Text.")
    print("Choose from the following functionalities:")
    print("1. Evaluate.")
    print("2. Predict.")
    print("Select either 1 or 2.")
    choice = input()
    choice_number = int(choice)

    if choice == 1:
        print("Started evaluation of " + test_filepath)
        predicted_val = marvalous.evaluate(test_filepath=test_filepath)
        print("Evaluation completed.")
    if choice == 2:
        print("Started prediction of " + test_filepath)
        predicted_val = marvalous.predict(test_filepath=test_filepath)
        print("Prediction completed.")
        predicted_val_df = pd.DataFrame(predicted_val)
        predicted_val_filepath = test_filepath.replace(".xlsx", "_predicted_val.csv")
        predicted_val_df.to_csv(predicted_val_filepath, header=None, index=False)
        print("Predicted label of text written to file: " + predicted_val_filepath)